#import "teapot.h"

#import <OpenGL/gl.h>

extern void RenderScene(void);
extern void SetupScene(void);
extern void Reshape(int w, int h);
extern void SpecialKey(int key, int x, int y);


extern GLfloat g_theta,g_phi;

@implementation teapot

@synthesize window;

-( void ) drawRect : ( NSRect ) rect {
	RenderScene();
 [[ self openGLContext ] flushBuffer ];
}

-( id ) initWithFrame : ( NSRect ) frame
{
    NSLog( @"myOpenGLView::initWithFrame" );
    GLuint attributes[] =
    {
        NSOpenGLPFAWindow,         // choose among pixelformats capable of rendering to windows
        NSOpenGLPFAAccelerated,    // require hardware-accelerated pixelformat
        NSOpenGLPFADoubleBuffer,   // require double-buffered pixelformat
        NSOpenGLPFAColorSize, 24,  // require 24 bits for color-channels
        NSOpenGLPFAAlphaSize, 8,   // require an 8-bit alpha channel
        NSOpenGLPFADepthSize, 24,  // require a 24-bit depth buffer
        NSOpenGLPFAMinimumPolicy,  // select a pixelformat which meets or exceeds these requirements
        0
    };
    NSOpenGLPixelFormat * pixelformat = [[ NSOpenGLPixelFormat alloc ] initWithAttributes: ( NSOpenGLPixelFormatAttribute * ) attributes ];
    if( pixelformat == nil )
    {
        NSLog( @"No valid OpenGL pixel format" );
    }
    // now init the window
    return( self = [ super initWithFrame: frame pixelFormat: [ pixelformat autorelease ] ] );
}

- (void) setup {
	NSOpenGLContext *glcontext;
	NSLog(@"Prepare called");
	glcontext=[self openGLContext];
	[glcontext makeCurrentContext];
	SetupScene();
}

- (void)reshape {
	NSRect rect = window.frame;
	NSRect frame = NSMakeRect(0, 0, (int)rect.size.width,(int)rect.size.height);
	[self setFrame:frame];

	NSLog(@"Re-shape %f %f",rect.size.width,rect.size.height);
	
    Reshape(( GLsizei ) rect.size.width, ( GLsizei ) rect.size.height );	
    //glViewport( 0, 0, ( GLsizei ) rect.size.width, ( GLsizei ) rect.size.height );
	[self setNeedsDisplay:YES];
}


- (void)mouseDown:(NSEvent *)event {
	NSPoint screenPoint=[NSEvent mouseLocation];
	pd=[event locationInWindow];
	NSLog(@"Mouse down  (%f, %f) Click %d  Screen (%f,%f)",pd.x,pd.y, [event clickCount],screenPoint.x,screenPoint.y);
}

- (void)mouseUp:(NSEvent *)event {
	NSLog(@"Mouse up");
}

- (void)mouseDragged:(NSEvent *)event {
	pd=[event locationInWindow];
	NSLog(@"Mouse dragged coord  (%f, %f) Click %d",pd.x,pd.y, [event clickCount]);
}

- (void)mouseMoved:(NSEvent *)event {  // must have this set to NSWindow
	pd=[event locationInWindow];
	NSLog(@"Mouse moved (%f, %f) Click %d",pd.x,pd.y, [event clickCount]);
}


- (BOOL)acceptsFirstResponder  // allowd keyboard events to be used
{
    return YES;
}

- (void)keyDown:(NSEvent *)event
{
    BOOL handled = NO;
    NSString  *characters;
	int code = [event keyCode];
    // get the pressed key
    characters = [event charactersIgnoringModifiers];
	const char *sp;
	if((sp = [characters cStringUsingEncoding:NSASCIIStringEncoding]) != NULL){
		NSLog(@"Key down %d key[%s]",code,sp);
	}
	else {
		SpecialKey(code,0,0);
		RenderScene();
		[[ self openGLContext ] flushBuffer ];

		NSLog(@"Key down %d",code);
	}
    
    // is the "r" key pressed?
    if ([characters isEqual:@"r"]) {
		// Yes, it is
		handled = YES;
		NSLog(@"r key ");
    }
    if (!handled)
		[super keyDown:event];
    
}


@end
